package watcher

import (
	"context"
	"fmt"
	coach_client "gitea.timerzz.com/kedaya_haitao/common/pkg/coach-client"
	"gitea.timerzz.com/kedaya_haitao/pusher/kitex_gen/push"
	"gitea.timerzz.com/kedaya_haitao/pusher/rpc/pusher"
	"github.com/golang/glog"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
	"haitao_watcher/pkg/model"
	"math/rand/v2"
	"time"
)

type CoachOutlet struct {
	cfg    *model.WatchInfo
	db     *gorm.DB
	fCtx   context.Context
	ctx    context.Context
	cancel context.CancelFunc
	client *coach_client.Client
}

func NewCoachWatcher(ctx context.Context, client *coach_client.Client, cfg *model.WatchInfo, db *gorm.DB) *CoachOutlet {
	return &CoachOutlet{
		fCtx:   ctx,
		client: client,
		cfg:    cfg,
		db:     db,
	}
}

func (c *CoachOutlet) Option() model.WatchInfo {
	return *c.cfg
}

func (c *CoachOutlet) SetOption(cfg model.WatchInfo) {
	c.cfg = &cfg
}

func (c *CoachOutlet) Cancel() {
	c.cancel()
}

func (c *CoachOutlet) Restart() {
	c.Cancel()
	go c.Watch()
}

func (c *CoachOutlet) Watch() {
	c.ctx, c.cancel = context.WithCancel(c.fCtx)
	c.cfg.Watch, c.cfg.Orderable = true, false
	if err := c.db.Model(c.cfg).Where("uid", c.cfg.Uid).Updates(map[string]interface{}{"watch": true, "orderable": false}).Error; err != nil {
		glog.Errorf("watch update db err: %v", err)
		err = nil
	}

	if c.doWatch() {
		c.Push()
		return
	}

	ticker := time.NewTicker(time.Second * 150)
	defer ticker.Stop()

	for {
		select {
		case <-ticker.C:
			// 加一些干扰
			salt := rand.IntN(70)
			time.Sleep(time.Second * time.Duration(salt))
			if c.doWatch() {
				c.Push()
				return
			}
		case <-c.ctx.Done():
			return
		}
	}
}
func (c *CoachOutlet) doWatch() bool {
	// 如果没有详情，先获取详情
	if !c.cfg.HasDetail {
		p, err := c.client.RequestProductDetail(c.ctx, c.cfg.Pid)
		if err != nil {
			glog.Errorf("获取coach %s 详情失败:%v", c.cfg.Pid, err)
			return false
		}
		c.cfg.Uid = fmt.Sprintf("coachOutlet_%s", p.Id)
		c.cfg.Name = p.Name
		c.cfg.Brand = p.Brand
		c.cfg.Website = model.CoachOutlet
		c.cfg.Orderable = p.Inventory.Orderable
		c.cfg.Link = fmt.Sprintf("https://www.coachoutlet.com%s", p.Url)
		c.cfg.HasDetail = true
		c.cfg.AllocationResetDate = p.Inventory.AllocationResetDate
		if c.cfg.Orderable && c.cfg.AllocationResetDate.IsZero() {
			c.cfg.AllocationResetDate = time.Now()
		}
		if c.cfg.Orderable {
			c.cfg.Watch = false
		}
		err = c.db.Clauses(clause.OnConflict{
			Columns:   []clause.Column{{Name: "uid"}},
			UpdateAll: true,
		}).Create(c.cfg).Error
		if err != nil {
			glog.Errorf("保存 %s detail失败:%v", c.cfg.Uid, err)
			c.cfg.HasDetail = false
			return false
		}
		if c.cfg.Orderable {
			return true
		}
	}

	inventory, err := c.client.RequestInventory(c.ctx, c.cfg.Pid)
	if err != nil {
		glog.Warningf("获取coach %s 库存失败:%v", c.cfg.Pid, err)
		return false
	}
	c.cfg.Orderable = inventory.Orderable
	if !inventory.AllocationResetDate.IsZero() {
		c.cfg.AllocationResetDate = inventory.AllocationResetDate
	}
	if c.cfg.Orderable {
		c.cfg.Watch = false
	}

	if err = c.db.Model(&model.WatchInfo{}).Where("uid = ?", c.cfg.Uid).Updates(
		map[string]interface{}{
			"orderable":             c.cfg.Orderable,
			"allocation_reset_date": c.cfg.AllocationResetDate,
			"watch":                 c.cfg.Watch,
			"updated_at":            time.Now(),
		}).Error; err != nil {
		glog.Errorf("更新数据库失败：%v", err)
		c.cfg.Orderable, c.cfg.Watch = false, true
	}
	return c.cfg.Orderable
}

func (c *CoachOutlet) Push() {
	resp, err := pusher.Push(c.ctx, &push.PushReq{
		Ids:     c.cfg.ToPusherIds(),
		Title:   "coach 补货",
		Content: fmt.Sprintf("coach 商品 %s 补货\n商品名：%s\n链接：%s", c.cfg.Pid, c.cfg.Name, c.cfg.Link),
	})
	if err != nil {
		glog.Errorf("消息推送失败：%v", err)
	}
	if resp.Code != 0 {
		glog.Errorf("消息推送失败:%s", resp.Msg)
	}
}
